# This file is part of caucase
# Copyright (C) 2017  Nexedi
#     Alain Takoudjou <alain.takoudjou@nexedi.com>
#     Vincent Pelletier <vincent@nexedi.com>
#
# caucase is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# caucase is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with caucase.  If not, see <http://www.gnu.org/licenses/>.
import json

class CertificateAuthority(object):
  def __init__(self, storage, ca_life_time, digest_list, crt_constraints...):
    self._storage = storage
    self.reload()

  def reload(self):
    """
    Refresh instance's knowledge of database content
    (as storage house-keeping may/will happen outside our control)
    """
    self._ca_key_pairs_list = storage.getCAKeyPairList()
    if not self._ca_key_pairs_list:
      self.createCAKeyPair()
      self._ca_key_pairs_list = storage.getCAKeyPairList()
      assert self._ca_key_pairs_list

  def createCAKeyPair(self):
    """
    Create a new ca key + certificate pair
    """
    # generate CA key pair
    self._storage.storeCAKeyPair(key_pair)

  def getPendingCertificateRequestList(self):
    return self._storage.getPendingCertificateRequestList()

  def createCertificateSigningRequest(self, csr):
    """
    Sanity-check CSR, stores it and generates a unique signing request
    identifier (crt_id).
    """
    # Check number of already-pending signing requests
    # Check if csr is self-signed
    # Check it has a CN (?)
    # Check its extensions
    # more ?
    return self._storage.storeCertificateSigningRequest(csr)

  def deletePendingCertificateRequest(self, crt_id):
    """
    Reject a pending certificate signing request.
    """
    self._storage.deletePendingCertificateRequest(crt_id)

  def getCertificateSigningRequest(self, crt_id):
    """
    Retrieve the content of a pending signing request.
    """
    return self._storage.getCertificateSigningRequest(crt_id)

  def createCertificate(self, crt_id):
    csr = self.getCertificateSigningRequest(crt_id)
    # Apply extensions (ex: "not a certificate", ...)
    # Generate a certificate from the CSR
    # Sign the certificate with the current CA key
    self._storage.storeCertificate(crt_id, crt)
    return crt

  def getCertificate(self, crt_id):
    return self._database.getCertificate(crt_id)

  def getCACertificate(self):
    """
    Return current CA certificate
    """
    return self._ca_key_pairs_list[-1].crt

  def getValidCACertificateChain(self):
    """
    Return the ca certificate chain for all valid certificates
    """
    result = []
    iter_key_pair = iter(self._ca_key_paid_list)
    previous_key_pair = iter_key_pair.next()
    for key_pair in iter_key_pair:
      result.append(utils.wrap({
        'old': previous_key_pair.crt,
        'new': key_pair.crt,
      }, previous_key_pair.key, self.digest_list))
    return result

  def revokeCertificate(self, wrapped_crt):
    payload = utils.unwrap(wrapped_crt, lambda x: x['crt'], self.digest_list)
    crt = payload['revoke_crt']
    self._storage.revokeCertificate(crt.getSerial(), crt.getNotAfterDate())

  def renew(self, wrapped_csr):
    payload = utils.unwrap(wrapped_crt, lambda x: x['crt'], self.digest_list)
    csr = payload['renew_csr']
    crt_id = self.createCertificateSigningRequest(csr)
    self.createCertificate(crt_id)
    return crt_id

  def getCertificateRevocationList(self):
    self._storage.getCertificateRevocationList()
